#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

// LED-Anzeige
#define STL0 PC5
#define STL1 PC4
#define STL2 PC3

#define SEG0 (1<<PD5)
#define SEG1 (1<<PD7)
#define SEG2 (1<<PD6)
#define SEG3 (1<<PD4)
#define SEG4 (1<<PD0)
#define SEG5 (1<<PD2)
#define SEG6 (1<<PD1)
#define SEG7 (1<<PD3)

#define A0 (SEG1 | SEG2 | SEG3 | SEG5 | SEG6 | SEG7)
#define A1 (SEG3 | SEG6 )
#define A2 (SEG1 | SEG2 | SEG4 | SEG6 | SEG7)
#define A3 (SEG1 | SEG3 | SEG4 | SEG6 | SEG7)
#define A4 (SEG3 | SEG4 | SEG5 | SEG6)
#define A5 (SEG1 | SEG3 | SEG4 | SEG5 | SEG7)
#define A6 (SEG1 | SEG2 | SEG3 | SEG4 | SEG5 | SEG7)
#define A7 (SEG3 | SEG6 | SEG7)
#define A8 (SEG1 | SEG2 | SEG3 | SEG4 | SEG5 | SEG6 | SEG7)
#define A9 (SEG1 | SEG3 | SEG4 | SEG5 | SEG6 | SEG7)

// Drehgeber
#define PHASE_A		(PINC & 1<<PC1)
#define PHASE_B		(PINC & 1<<PC0)
#define TASTER    ((PINC & 1<<PC2) >> PC2)

// Uhr
#define TEILER 260L        // debounce clock (256Hz = 4msec)

// Ausgang
#define RELAIS PB1

const char ziffer[12] = {A0, A1, A2, A3, A4, A5, A6, A7, A8, A9, 0};

void anzeige(int16_t zahl,char stelle);


volatile int16_t sekundenzaehler;
volatile uint8_t sekundentimer = 0; // Timer fr Sekundenzhler
volatile uint8_t ledtimer = 0;      // Timer fr LED-blinken
volatile char led_flag = 0;
volatile char wait_flag = 1;

// Timer-Interruptroutine
SIGNAL (SIG_OUTPUT_COMPARE1A) // WinAVR
// ISR(TIMER1_COMPA_vect) // Neuere Version von AVR-GCC
{
  // Sekundenzaehler
	if (!wait_flag)
	{
		if (sekundentimer > 48)
		{
			sekundenzaehler--;
			sekundentimer=0;
		}
		else
		sekundentimer++;
	}

  // Blinkgeber
	if (ledtimer > 12)
	{
		led_flag = ! led_flag;
		ledtimer = 0;
	}
	else
	ledtimer++;
}



int main(void)
{
	// Initialisierung LED-Anzeige
	PORTB = 0x00;
	PORTC = (1 << PC0) | (1 << PC1) | (1 << PC2); // Drehgeber-Pullup 
	DDRD = 0xFF;
	DDRC |= (1 << PC3) | (1 << PC4) | (1 << PC5); // LED-+
  DDRB |= (1 << RELAIS);  // Relais

	sekundenzaehler=00;
	char stelle=0; 
	// Ende Init LED-Anzeige


	// Eingabe Taster und Drehgeber
	// Initialisieren
	int8_t neu = 0;
	int8_t alt = 0;
	int8_t enc_delta = 0;
	int8_t diff = 0;
	char t0=0b01010101;
	char tast_flag=0;
	

	neu = 0;
	if( PHASE_A )
	neu = 3;
	if( PHASE_B )
	neu ^= 1;					  // Gray -> Binr

	alt = neu;					// Startwert
	enc_delta = 0;
	// Ende Init Drehgeber



	// Initialisierung Uhr/Sekundenzhler
	TCCR1B = (1<<WGM12) | (1<<CS10) | (1 << CS12);      // Prescaler/Vorteiler = 1024, clear on compare
	OCR1A = TEILER;                        // Output Compare Register
	TCNT1 = 0;                             // Timer startet mit 0
	TIMSK1 = 1<<OCIE1A;                    // beim Vergleichswertes Compare Match                    
	sei();                                 // Interrupt (SIG_OUTPUT_COMPARE1A)
	// Ende Uhr






	while(1)
	{
		// Drehgeber dekodieren (nur im Wartemodus!)
		if (wait_flag)
		{
			neu = 0;
			if( PHASE_B )
			neu = 3;
			if( PHASE_A )
			neu ^= 1;					    // convert gray to binary

			diff = alt - neu;			// difference last - new
			if( diff & 1 ){				// bit 0 = value (1)
				alt = neu;					// store new as next last
				enc_delta += (diff & 2) - 1;		// bit 1 = direction (+/-)
			}

			sekundenzaehler += enc_delta;
			enc_delta=0;
		} // Ende Drehgeber


		if (sekundenzaehler<0) sekundenzaehler=0;
		if (sekundenzaehler>999) sekundenzaehler=999;
		if (sekundenzaehler==0) wait_flag=1;

    // LED-Ausgabe: Im Wartemodus blinkend, im Zhlmodus durchgehend 
		if (led_flag | !wait_flag)
		{
			anzeige(sekundenzaehler,stelle);
			stelle++;
			if (stelle>2) stelle=0;
			_delay_ms(2);
		}
		else
		{
			PORTD = 0xFF; // LEDs aus
		}

		
		// Taster abfragen und entprellen
		t0 <<= 1;
		t0 |= TASTER;
		if (t0 == 0xFF && !tast_flag) 
		{
			wait_flag = !wait_flag;
			tast_flag = 1;
		}
		if (t0 == 0)
		tast_flag = 0;


		// Ausgabe fr Relais (nur wenn nicht im Wartemodus)
		if (!wait_flag)
		  PORTB |= (1 << RELAIS); // LED/Relais an
		else
		  PORTB &= ~(1 << RELAIS); // LED/Relais aus

	} // Ende loop

	return 0;
}

// LED-Ausgabe einzelnder Ziffern
void anzeige(int16_t zahl,char stelle)
{
	uint8_t s = 0;
	switch (stelle)
	{
	case 0:
		PORTC &= ~(1 << STL2);
		s =  zahl / 100;
    if (s == 0) s = 10;
		PORTD = ~ziffer[s];
		PORTC |=  (1 << STL0);
		break;

	case 1:
		PORTC &= ~(1 << STL0);
		s = (zahl % 100)/10;
		if (s > 9) s = 10;
		PORTD = ~ziffer[s];
		PORTC |=  (1 << STL1);
		break;

	case 2: 
		PORTC &= ~(1 << STL1);
		s = (zahl % 10);
		PORTD = ~ziffer[s];
		PORTC |=  (1 << STL2);
	}

	return;
}

